"""credits.py - a context for displaying a game's credits
"""

import os
import random

import gsdl2

import context
import settings


class Credits(context.Context):
    
    def __init__(self):
        context.Context.__init__(self, gsdl2.time.FixedDriver(self.update, settings.time_step))
        self.clock.new_schedule(self.draw, 1.0 / 30.0)

        self.bgcolor = 0, 0, 0

        font_name, font_size = settings.font_info['intro']
        self.font = gsdl2.font.Font(os.path.join(settings.font_dir, font_name), 144)
        self.font_color = gsdl2.Color('gold')

        self.credits = None
        self.credits_rect = None
        self.elapsed = 0.0
        self.update(0)

    def update(self, dt):
        self.elapsed -= dt
        if self.elapsed <= 0.0:
            choice = random.choice
            choices = True, False
            self.elapsed += 1.0
            self.font.set_bold(choice(choices))
            self.font.set_italic(choice(choices))
            self.font.set_outline(choice(choices))
            self.font.set_underline(choice(choices))
            self.credits = self.font.render('GUMM', self.font_color)
            self.credits_rect = self.credits.get_rect(center=settings.screen_rect.center)
        self.get_events()
    
    def draw(self, sched):
        screen = settings.screen
        screen.fill(self.bgcolor)
        screen.blit(self.credits, self.credits_rect)
        gsdl2.display.flip()

    def on_key_down(self, e):
        self.on_quit()

    def on_mouse_button_down(self, e):
        self.on_quit()

    def on_quit(self):
        self.pop()
