"""intro.py - a context for displaying a splash screen, slideshow,
movie, etc.
"""

import os
import random

import gsdl2

import context
import mainmenu
import settings
from vec2d import Vec2d


class Intro(context.Context):
    
    def __init__(self):
        context.Context.__init__(self, gsdl2.time.FixedDriver(self.update, settings.time_step))
        self.clock.new_schedule(self.draw, 1.0 / 30.0)

        self.bgcolor = gsdl2.Color('darkgreen')

        self.font = None
        self.countdown = None
        self.text = None
        self.text_img = None
        self.text_rect = None
        self.dx = None
        self.dy = None
        self.old_pos = None
        self.old_dxy = None

    def enter(self):
        font_name, font_size = settings.font_info['intro']
        self.font = gsdl2.font.Font(os.path.join(settings.font_dir, font_name), font_size)
        self.countdown = 10.99999  # float, seconds
        self.text = ''
        self.text_rect = gsdl2.Rect(3, 3, 1, 1)
        self.update_text()
        self.dx = random.choice((-2, -2, -1, 1, 2, 3))
        self.dy = random.choice((-3, -2, -1, 1, 2, 3))
        self.old_pos = Vec2d(self.text_rect.topleft)
        self.old_dxy = Vec2d(self.dx, self.dy)
    
    def update(self, dt):
        self.countdown -= dt
        if int(self.countdown) <= 0:
            self.action_main_menu()
        
        self.update_text()
        text_rect = self.text_rect
        self.old_pos[:] = text_rect.topleft
        self.old_dxy[:] = self.dx, self.dy
        text_rect.x += self.dx
        text_rect.y += self.dy
        screen_rect = settings.screen_rect
        if not screen_rect.contains(text_rect):
            if text_rect.x < 0 or text_rect.right >= screen_rect.right:
                self.dx *= -1
            if text_rect.y < 0 or text_rect.bottom >= screen_rect.bottom:
                self.dy *= -1
            self.text_rect.clamp_ip(screen_rect)
    
    def update_text(self):
        self.get_events()
        new_text = 'Intro (Wait {0:d} or Click)'.format(int(self.countdown))
        if new_text == self.text:
            return
        self.text = new_text
        self.text_img = self.font.render(new_text, gsdl2.Color('white'))
        self.text_rect.size = self.text_img.get_size()
    
    def draw(self, sched):
        interp = sched.interp
        x, y = self.old_pos + self.old_dxy * interp
        interp_pos = round(x), round(y)

        screen = settings.screen
        screen.fill(self.bgcolor)
        screen.blit(self.text_img, interp_pos)
        gsdl2.display.flip()
    
    def action_main_menu(self):
        self.pop()
        self.push(mainmenu.MainMenu())
    
    def on_mouse_button_down(self, e):
        self.action_main_menu()

    def on_quit(self):
        self.pop()
