"""mainmenu.py - a context for running the application's main menu
"""

import os

import gsdl2

import context
import credits
import game
import settings


class ClickButton(object):
    
    def __init__(self, text_img, y, action):
        self.image = text_img
        self.rect = text_img.get_rect(y=y, centerx=settings.screen_rect.centerx)
        self.action = action
    
    def click(self, mouse_pos):
        if self.rect.collidepoint(mouse_pos):
            self.action()


class MainMenu(context.Context):
    
    def __init__(self):
        context.Context.__init__(self, gsdl2.time.FixedDriver(self.update, settings.time_step))
        self.clock.new_schedule(self.draw, 1.0 / 30.0)

        self.bgcolor = gsdl2.Color('darkgreen')
    
    def enter(self):
        font_name, font_size = settings.font_info['mainmenu']
        font = gsdl2.font.Font(os.path.join(settings.font_dir, font_name), font_size)
        screen_centery = settings.screen_rect.centery
        text_height = font.get_height()
        
        self.buttons = []
        
        n = -1
        text_img = font.render('PLAY GAME', gsdl2.Color('white'))
        self.buttons.append(ClickButton(
            text_img, screen_centery + n * text_height, self.action_play_game))
        
        n += 1
        text_img = font.render('CREDITS', gsdl2.Color('white'))
        self.buttons.append(ClickButton(
            text_img, screen_centery + n * text_height, self.action_show_credits))
        
        n += 1
        text_img = font.render('QUIT GAME', gsdl2.Color('white'))
        self.buttons.append(ClickButton(
            text_img, screen_centery + n * text_height, self.action_quit_game))
    
    def exit(self):
        ## Outro?
        pass
    
    def action_play_game(self):
        self.push(game.Game())
    
    def action_show_credits(self):
        self.push(credits.Credits())
    
    def action_quit_game(self):
        self.pop()
    
    def update(self, dt):
        self.get_events()
    
    def draw(self, sched):
        screen = settings.screen
        screen.fill(self.bgcolor)
        for b in self.buttons:
            screen.blit(b.image, b.rect)
        gsdl2.display.flip()
    
    def on_mouse_button_down(self, e):
        for b in self.buttons:
            b.click(e.pos)
