import gsdl2

import settings
import weapon


class Player(object):

    size = None
    zero = None

    def __init__(self):
        if Player.size is None:
            w = settings.resolution[0] // 50
            Player.size = w, w
            Player.zero = settings.playfield_rect.center

        self.x, self.y = Player.zero
        self.positions = []
        self.positions.append((self.x, self.y))
        self.dx, self.dy = 0, 0
        surf = gsdl2.Surface(Player.size)
        surf.fill(gsdl2.Color('cornflowerblue'))
        self.texture = gsdl2.Texture(gsdl2.display.get_renderer(), surf)
        self.rect = self.texture.get_rect(center=Player.zero, size=Player.size)
        self.interp_rect = self.rect.copy()
        self.speed = 5.0
        self.weapon = weapon.FriendlyWeapon(self, (0,), settings.player_shots_per_group)

    def set_pos(self, x, y):
        self.rect.center = x, y
        if not settings.playfield_rect.contains(self.rect):
            self.rect.clamp_ip(settings.playfield_rect)
            x, y = self.rect.center
        self.x = x
        self.y = y
        self.positions.append((x, y))

    def move_pos(self, dx, dy):
        self.set_pos(self.x + dx, self.y + dy)

    def gain_weapon(self):
        return self.weapon.gain_port()

    def lose_weapon(self):
        return self.weapon.lose_port()

    def draw_rect(self, interp):
        dx = self.dx
        dy = self.dy
        x, y = self.rect.topleft
        rect = self.interp_rect
        rect.topleft = x - dx + dx * interp, y - dy + dy * interp
        return rect

    def update(self, dt):
        # save old pos for use in interpolation
        x1, y1 = self.positions[0]
        x2, y2 = self.positions[-1]
        self.dx = x2 - x1
        self.dy = y2 - y1
        del self.positions[:-1]
        self.weapon.update(dt)
