"""gummworld2.py - startup and runtime settings file

Usage
=====
Settings in this file should always be accessed like so:
    
    import settings
    if settings.whatever: ...
    settings.whatever = value

This insures that runtime changes will be noticed by other modules that import
the settings module. Of course, one can always make local copies of the values
if static behavior is desired.
"""

import sys

import gsdl2


# Tunables
if 'pypy' in sys.executable:
    # max objects allowed in game system; Intel i7 w/ pypy can handle about 3000
    max_objects_allowed = 3000
else:
    # max objects allowed in game system; Intel i7 w/ Python 2.7 can handle about 300
    max_objects_allowed = 300

(enemy_rate,                # one wave per N seconds
 enemy_shots_per_group,     # num auto-shots per pulse
 enemy_fire_rate,           # num seconds per weapon pulse
 player_shots_per_group,    # num auto-shots per pulse
 player_fire_rate,          # num seconds per weapon pulse
 reward_rate,               # num hits for player to gain a weapon
 penalty_rate,              # num hits for player to lose a weapon
 end_game_bonus_per_wave,   # increase num enemies per wave when active ports are maxed
 ) = (
    (6.9, 2, 2.9, 3, 1.1, 2, 2, 10),
    (7.0, 2, 3.0, 3, 1.0, 3, 1, 10),
)[1]    # <-- pick one; 0=hard; 1=easier

# CPU
time_step = 1.0 / 30.0

# Display - None values are computed at runtime
resolution = 800, 800
display_flags = 0  # gsdl2.sdlconstants.SDL_WINDOW_FULLSCREEN
screen = None
screen_rect = None
playfield_rect = None

# Graphics
image_dir = 'data/image'
font_dir = 'data/font'
font_info = dict(
    intro=('Boogaloo.ttf', 55),
    mainmenu=('CherryCreamSoda.ttf', 44),
    score=('Bubblegum_Sans.ttf', 88),
)

# Sound
sound_dir = 'data/sound'
music_dir = 'data/music'
play_music = True

# Misc
SONG_FINISHED = 1  # music.set_endevent(code=SONG_FINISHED)

# Debug
profile = False             # run cProfile if True

# Custom
import os
os.environ['SDL_VIDEO_CENTERED'] = '1'
bypass_menu = None

# Gummbum's development preferences... remove or add a case for your own.
who = 'Gumm'
try:
    if who == 'Gumm':
        print('## CUSTOM SETTINGS FOR Gumm (see settings.py)')
        # See main.py for the effect of bypass_menu:
        bypass_menu = (None, 'game', 'credits')[0]
        map_size = (150, 150)
except:
    pass

del os, who, gsdl2
